//! cassiopeia file format

mod lexer;
mod parser;

pub(crate) use lexer::{LineLexer, LineToken, Token};
pub(crate) use parser::LineCfg;

use crate::TimeFile;
use std::{fs::File, io::Read};

/// Load a file from disk and parse it into a
/// [`TimeFile`](crate::TimeFile)
pub fn load_file(path: &str) -> Option<TimeFile> {
    let mut f = File::open(path).ok()?;
    let mut content = String::new();
    f.read_to_string(&mut content).ok()?;

    let mut lines: Vec<String> = content.split("\n").map(|l| l.to_owned()).collect();

    Some(
        lines
            .iter_mut()
            .map(|line| lexer::lex(line))
            .map(|lex| parser::parse(lex))
            .filter(|line| line.valid())
            .fold(TimeFile::default(), |file, line| file.append(line)),
    )
}
